import{a as x,g as k,c as o,o as r,b as l,l as p,m as t,F as w,B as C,k as f,t as S,aG as v,aD as $,bc as y,aO as g,aH as O,aI as j,h as n,b6 as B,aT as i,aE as m}from"./Bq-XgLEJ.js";const U={wrapper:{base:"flex items-center align-center text-center w-full",horizontal:"flex-row",vertical:"flex-col"},container:{base:"font-medium text-gray-700 dark:text-gray-200 flex",horizontal:"mx-3 whitespace-nowrap",vertical:"my-2"},border:{base:"flex border-gray-200 dark:border-gray-800",horizontal:"w-full",vertical:"h-full",size:{horizontal:{"2xs":"border-t",xs:"border-t-[2px]",sm:"border-t-[3px]",md:"border-t-[4px]",lg:"border-t-[5px]",xl:"border-t-[6px]"},vertical:{"2xs":"border-s",xs:"border-s-[2px]",sm:"border-s-[3px]",md:"border-s-[4px]",lg:"border-s-[5px]",xl:"border-s-[6px]"}},type:{solid:"border-solid",dotted:"border-dotted",dashed:"border-dashed"}},icon:{base:"flex-shrink-0 w-5 h-5"},avatar:{base:"flex-shrink-0",size:"2xs"},label:"text-sm",default:{size:"2xs"}},s=$(m.ui.strategy,m.ui.divider,U),_=k({components:{UIcon:g,UAvatar:y},inheritAttrs:!1,props:{label:{type:String,default:null},icon:{type:String,default:null},avatar:{type:Object,default:null},size:{type:String,default:()=>s.default.size,validator(e){return Object.keys(s.border.size.horizontal).includes(e)||Object.keys(s.border.size.vertical).includes(e)}},orientation:{type:String,default:"horizontal",validator:e=>["horizontal","vertical"].includes(e)},type:{type:String,default:"solid",validator:e=>["solid","dotted","dashed"].includes(e)},class:{type:[String,Object,Array],default:()=>""},ui:{type:Object,default:()=>({})}},setup(e){const{ui:a,attrs:d}=O("divider",j(e,"ui"),s),u=n(()=>B(i(a.value.wrapper.base,a.value.wrapper[e.orientation]),e.class)),c=n(()=>i(a.value.container.base,a.value.container[e.orientation])),b=n(()=>i(a.value.border.base,a.value.border[e.orientation],a.value.border.size[e.orientation][e.size],a.value.border.type[e.type]));return{ui:a,attrs:d,wrapperClass:u,containerClass:c,borderClass:b}}});function A(e,a,d,u,c,b){const z=g,h=y;return r(),o("div",v({class:e.wrapperClass},e.attrs),[l("div",{class:t(e.borderClass)},null,2),e.label||e.icon||e.avatar||e.$slots.default?(r(),o(w,{key:0},[l("div",{class:t(e.containerClass)},[C(e.$slots,"default",{},()=>[e.label?(r(),o("span",{key:0,class:t(e.ui.label)},S(e.label),3)):e.icon?(r(),f(z,{key:1,name:e.icon,class:t(e.ui.icon.base)},null,8,["name","class"])):e.avatar?(r(),f(h,v({key:2},{size:e.ui.avatar.size,...e.avatar},{class:e.ui.avatar.base}),null,16,["class"])):p("",!0)])],2),l("div",{class:t(e.borderClass)},null,2)],64)):p("",!0)],16)}const D=x(_,[["render",A]]);export{D as _};
