const e={width:800,height:400,priceScale:{borderColor:"rgba(197, 203, 206, 1)"},timeScale:{borderColor:"rgba(197, 203, 206, 1)",timeVisible:!0,secondsVisible:!1}},r={chart:{layout:{background:{color:"#ffffff"},textColor:"rgba(33, 56, 77, 1)"},grid:{vertLines:{color:"#f1f1f1",visible:!0},horzLines:{color:"#f1f1f1",visible:!0}},priceScale:{borderColor:"rgba(197, 203, 206, 0.6)"},timeScale:{borderColor:"rgba(197, 203, 206, 0.6)",timeVisible:!0,secondsVisible:!1}}},o={chart:{layout:{background:{color:"#333333"},textColor:"#D1D5DB"},grid:{vertLines:{color:"#525252",visible:!0},horzLines:{color:"#525252",visible:!0}},priceScale:{borderColor:"#525252"},timeScale:{borderColor:"#525252",timeVisible:!0,secondsVisible:!1}}};export{o as d,r as l,e as s};
