import{g as n,a8 as r,b4 as c,c as i,o as p}from"./Bq-XgLEJ.js";const _=n({__name:"index",setup(u){const a=r(),e=c(),t=Object.keys(e.tabs);if(t.length>0){const s=t[0],o=e.tabs[s];a.push({path:`/live/${o.id}`})}else e.addTab();return(s,o)=>(p(),i("div"))}});export{_ as default};
