import{_ as t}from"./S3kW5emf.js";import{g as n,c as s,k as a,o}from"./Bq-XgLEJ.js";const p=["innerHTML"],m=n({__name:"Logs",props:{logs:{}},setup(c){return(e,_)=>{const r=t;return e.logs.length?(o(),s("pre",{key:0,class:"whitespace-pre-line rounded border dark:border-gray-600 bg-gray-50 dark:bg-gray-700 select-text text-base dark:text-gray-300 w-full px-4 sm:px-6 py-2",innerHTML:e.logs},null,8,p)):(o(),a(r,{key:1}))}}});export{m as _};
