import{a as t,c as o,o as s,B as a,b as n}from"./Bq-XgLEJ.js";const d={},c={class:"text-center rounded border-2 border-dashed border-gray-300 dark:border-gray-700 bg-gray-50 dark:bg-gray-800 py-4 text-sm"};function _(r,e){return s(),o("div",c,[a(r.$slots,"default",{},()=>[e[0]||(e[0]=n("span",{class:"text-gray-400"}," Empty ",-1))])])}const p=t(d,[["render",_]]);export{p as _};
