.spotlight[data-v-00b6b518]{background:linear-gradient(45deg,#00dc82,#36e4da 50%,#0047e1);bottom:-30vh;filter:blur(20vh);height:40vh}.gradient-border[data-v-00b6b518]{-webkit-backdrop-filter:blur(10px);backdrop-filter:blur(10px);border-radius:.5rem;position:relative}@media (prefers-color-scheme:light){.gradient-border[data-v-00b6b518]{background-color:#ffffff4d}.gradient-border[data-v-00b6b518]:before{background:linear-gradient(90deg,#e2e2e2,#e2e2e2 25%,#00dc82,#36e4da 75%,#0047e1)}}@media (prefers-color-scheme:dark){.gradient-border[data-v-00b6b518]{background-color:#1414144d}.gradient-border[data-v-00b6b518]:before{background:linear-gradient(90deg,#303030,#303030 25%,#00dc82,#36e4da 75%,#0047e1)}}.gradient-border[data-v-00b6b518]:before{background-size:400% auto;border-radius:.5rem;bottom:0;content:"";left:0;-webkit-mask:linear-gradient(#fff 0 0) content-box,linear-gradient(#fff 0 0);mask:linear-gradient(#fff 0 0) content-box,linear-gradient(#fff 0 0);-webkit-mask-composite:xor;mask-composite:exclude;opacity:.5;padding:2px;position:absolute;right:0;top:0;transition:background-position .3s ease-in-out,opacity .2s ease-in-out;width:100%}.gradient-border[data-v-00b6b518]:hover:before{background-position:-50% 0;opacity:1}.fixed[data-v-00b6b518]{position:fixed}.left-0[data-v-00b6b518]{left:0}.right-0[data-v-00b6b518]{right:0}.z-10[data-v-00b6b518]{z-index:10}.z-20[data-v-00b6b518]{z-index:20}.grid[data-v-00b6b518]{display:grid}.mb-16[data-v-00b6b518]{margin-bottom:4rem}.mb-8[data-v-00b6b518]{margin-bottom:2rem}.max-w-520px[data-v-00b6b518]{max-width:520px}.min-h-screen[data-v-00b6b518]{min-height:100vh}.w-full[data-v-00b6b518]{width:100%}.flex[data-v-00b6b518]{display:flex}.cursor-pointer[data-v-00b6b518]{cursor:pointer}.place-content-center[data-v-00b6b518]{place-content:center}.items-center[data-v-00b6b518]{align-items:center}.justify-center[data-v-00b6b518]{justify-content:center}.overflow-hidden[data-v-00b6b518]{overflow:hidden}.bg-white[data-v-00b6b518]{--un-bg-opacity:1;background-color:#fff;background-color:rgb(255 255 255/var(--un-bg-opacity))}.px-4[data-v-00b6b518]{padding-left:1rem;padding-right:1rem}.px-8[data-v-00b6b518]{padding-left:2rem;padding-right:2rem}.py-2[data-v-00b6b518]{padding-bottom:.5rem;padding-top:.5rem}.text-center[data-v-00b6b518]{text-align:center}.text-8xl[data-v-00b6b518]{font-size:6rem;line-height:1}.text-xl[data-v-00b6b518]{font-size:1.25rem;line-height:1.75rem}.text-black[data-v-00b6b518]{--un-text-opacity:1;color:#000;color:rgb(0 0 0/var(--un-text-opacity))}.font-light[data-v-00b6b518]{font-weight:300}.font-medium[data-v-00b6b518]{font-weight:500}.leading-tight[data-v-00b6b518]{line-height:1.25}.font-sans[data-v-00b6b518]{font-family:ui-sans-serif,system-ui,-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Helvetica Neue,Arial,Noto Sans,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol,Noto Color Emoji}.antialiased[data-v-00b6b518]{-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}@media (prefers-color-scheme:dark){.dark\:bg-black[data-v-00b6b518]{--un-bg-opacity:1;background-color:#000;background-color:rgb(0 0 0/var(--un-bg-opacity))}.dark\:text-white[data-v-00b6b518]{--un-text-opacity:1;color:#fff;color:rgb(255 255 255/var(--un-text-opacity))}}@media (min-width:640px){.sm\:px-0[data-v-00b6b518]{padding-left:0;padding-right:0}.sm\:px-6[data-v-00b6b518]{padding-left:1.5rem;padding-right:1.5rem}.sm\:py-3[data-v-00b6b518]{padding-bottom:.75rem;padding-top:.75rem}.sm\:text-4xl[data-v-00b6b518]{font-size:2.25rem;line-height:2.5rem}.sm\:text-xl[data-v-00b6b518]{font-size:1.25rem;line-height:1.75rem}}
