import{g as d,c as l,d as s,b as t,w as o,D as m,bD as c,q as _,o as p,e as r}from"./Bq-XgLEJ.js";import{_ as u}from"./BQPmH1-w.js";import{_ as g}from"./Bzgyw6a8.js";import{u as f}from"./CI3sxcT_.js";const x={class:"grid items-start lg:grid-cols-5"},b={class:"grid grid-cols-1 lg:col-span-4 bg-backdrop dark:bg-backdrop-dark"},k={class:"flex items-center justify-center select-none",style:{height:"calc(100vh - 4rem - 4px)"}},v={class:"text-center"},S={class:"mb-4"},D=d({__name:"index",setup(y){return f({title:"Strategies - Jesse"}),(n,e)=>{const a=_,i=u;return p(),l("section",x,[s(g),t("div",b,[t("div",k,[t("div",v,[e[4]||(e[4]=t("h1",{class:"font-semibold mb-16 text-5xl"}," Strategies ",-1)),t("div",S,[s(a,{icon:"i-heroicons-plus",onClick:e[0]||(e[0]=B=>("useTempStore"in n?n.useTempStore:m(c))().makeStrategy=!0)},{default:o(()=>e[1]||(e[1]=[r(" Create a new strategy ")])),_:1}),s(i,{class:"my-6 text-xs"},{default:o(()=>e[2]||(e[2]=[r(" OR ")])),_:1}),s(a,{icon:"i-heroicons-link",color:"gray",to:"https://jesse.trade/strategies",target:"_blank"},{default:o(()=>e[3]||(e[3]=[r(" Browse free and premium strategies ")])),_:1})])])])])])}}});export{D as default};
