import{g as f,c as n,o as i,F as u,p as g,b as c,t as d,v as b,x as m}from"./Bq-XgLEJ.js";import{f as y}from"./BZnfaEav.js";const h={class:"dark:bg-gray-700 bg-white grid grid-cols-1 gap-6 border dark:border-gray-600 rounded py-4 px-6"},p={class:"text-sm font-medium text-gray-500 dark:text-gray-400 truncate"},x={class:"text-sm font-semibold text-gray-900 dark:text-gray-100"},k=f({__name:"KeyValueTableSimple",props:{data:{}},setup(a){return(s,l)=>(i(),n("dl",h,[(i(!0),n(u,null,g(s.data,(r,e)=>(i(),n("div",{key:e,class:"flex justify-between items-center"},[c("div",p,d(r[0]),1),c("div",x,d(r[1]),1)]))),128))]))}}),w={width:800,height:380,crosshair:{mode:y.Normal}},C={chart:{layout:{background:{color:"#ffffff"},textColor:"rgba(33, 56, 77, 1)"},grid:{vertLines:{color:"#f1f1f1",visible:!1},horzLines:{color:"#f1f1f1",visible:!1}},priceScale:{borderColor:"rgba(197, 203, 206, 0.6)"},timeScale:{borderColor:"rgba(197, 203, 206, 0.6)",timeVisible:!0,secondsVisible:!1}}},L={chart:{layout:{background:{color:"#333333"},textColor:"#D1D5DB"},grid:{vertLines:{color:"#525252",visible:!1},horzLines:{color:"#525252",visible:!1}},priceScale:{borderColor:"#525252"},timeScale:{borderColor:"#525252",timeVisible:!0,secondsVisible:!1}}};function S(a,s={}){const l=e=>{const t=e;if(!t)return!1;const o=(t.tagName||"").toLowerCase();return!!(o==="input"||o==="textarea"||t.isContentEditable||t.getAttribute&&t.getAttribute("role")==="textbox")},r=e=>{if(s.onlyWhen&&!s.onlyWhen()||l(e.target)||e.metaKey||e.ctrlKey||e.altKey)return;const t=e.key.toLowerCase(),o=a[e.key]||a[t];o&&(e.preventDefault(),o(e))};b(()=>{window.addEventListener("keydown",r)}),m(()=>{window.removeEventListener("keydown",r)})}export{k as _,L as d,C as l,w as s,S as u};
