import{a as e,c as o,o as t,b as s,B as c}from"./Bq-XgLEJ.js";const n={},r={class:"py-10 mx-auto"},l={class:"mx-auto max-w-7xl px-2 md:px-8"},_={class:"max-w-[600px] mx-auto"};function d(a,m){return t(),o("div",r,[s("main",null,[s("div",l,[s("div",_,[c(a.$slots,"default")])])])])}const x=e(n,[["render",d]]);export{x as S};
